///////////////////////////////////////////////////////////////////////////////
//           ANALYSIS PROGRAM FOR CMAM SETUP                                 // 
//                                            Vicente García Távora Sept 2022//
///////////////////////////////////////////////////////////////////////////////
// 4*32 DSSD + 14*4 SiBall + 4* PADs + 8*GAGGs + CEPA4

//C,C++ Libraries
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <cstdlib>
#include <fstream>
#include <algorithm> // for std::find
#include <iterator> // for std::begin, std::end
// ROOT libraries
#include <TROOT.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TTree.h>
#include <TFile.h>
#include <TF1.h>
#include <TRandom3.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TColor.h>
#include <TStyle.h>
#include <TCutG.h>
using namespace std;

Int_t main(){

    cout << "Program started at:  " << endl; 
    system("/bin/date"); 
    cout << endl;
    
////////////////////////////////////////////////////////////
//            Read calibration parameters file            // 
////////////////////////////////////////////////////////////
// read calibration parameter file and store values in bidimensional arrays
// ={}{} fill all with 0
    
//     ifstream file("calibrations/multifit/calibration_parameters.txt");
//     
//      Double_t mval, nval;
//      Int_t adc, ch_adc;
//      Double_t m[17][16]; Double_t n[17][16];
//      
//      for (Int_t j=0; j<17; j++){
//          for(Int_t i=0; i<16; i++){
//              m[j][i]=0; n[j][i]=0;
//          }
//      }
//     
//     while (file >> mval >> nval >> adc >> ch_adc){
//      
//      m[adc][ch_adc] = mval;
//      n[adc][ch_adc] = nval;
//     }
//     file.close();
    
// with index as m[j][i] 
// adc=0 -> DSSD1_V[16]
// adc=1 -> DSSD1_H[16]
// adc=2 -> DSSD2_V[16]
// adc=3 -> DSSD2_H[16]
// adc=4 -> DSSD3_V[16]
// adc=5 -> DSSD3_H[16]
// adc=6 -> DSSD4_V[16]
// adc=7 -> DSSD4_H[16]   
// adc=8 -> Siball1[16]
// adc=9 -> Siball2[16]
// adc=10 -> Siball3[16]
// adc=11 -> Siball4[16]
// adc=12 -> PAD[4]
// adc=13 -> GAGG_ilong[8]
// adc=14 -> GAGG_ishort[8]
// adc=15 -> CEPA_ilong[8]
// adc=16 -> CEPA_ishort[8]
    
////////////////////////////////////////////////////////////
//               Empty histograms generator               // 
////////////////////////////////////////////////////////////

// Defining histogram name variable
    char DSSD1_Vname[40], DSSD1_Vname_histo[40];
    char DSSD1_Hname[40], DSSD1_Hname_histo[40];
    char PAD_name[40], PAD_name_histo[40];
    char GAGG_ilong_name[40], GAGG_ilong_name_histo[40], GAGG_ishort_name[40], GAGG_ishort_name_histo[40];
    
// Defining histogram arrays
    TH1F *DSSD1_V[16],*DSSD1_H[16]; // DSSDs
    TH1F *GAGG_ilong[4], *GAGG_ishort[4]; // GAGG long and short - MDPP16-QDC [0-7]
    TH1F *PAD[1]; // PADs
        
// Loop to create empty histograms
    cout << "Creating empty histograms" << endl;
    for (Int_t i=0; i<16;i++) {
        sprintf(DSSD1_Vname,"DSSD1_P%i",i);
        sprintf(DSSD1_Vname_histo,"DSSD1_P%i ; Channel ; Counts",i);
        DSSD1_V[i] = new TH1F(DSSD1_Vname, DSSD1_Vname_histo, 4096, 0, 4096);
        
        sprintf(DSSD1_Hname,"DSSD1_N%i",i);
        sprintf(DSSD1_Hname_histo,"DSSD1_N%i ; Channel ; Counts",i);
        DSSD1_H[i] = new TH1F(DSSD1_Hname, DSSD1_Hname_histo, 4096, 0, 4096);
        
        if (i < 1){
        sprintf(PAD_name,"PAD_%i",i);
        sprintf(PAD_name_histo,"PAD_%i ; Channel ; Counts",i);
        PAD[i] = new TH1F(PAD_name, PAD_name_histo, 4096, 0, 4096);
        }
        
        if (i < 4){
        sprintf(GAGG_ilong_name,"GAGG_ilong_%i",i);
        sprintf(GAGG_ilong_name_histo,"GAGG_ilong_%i ; Channel ; Counts",i);
        GAGG_ilong[i] = new TH1F(GAGG_ilong_name, GAGG_ilong_name_histo, 4096, 0, 65536);
        
        sprintf(GAGG_ishort_name,"GAGG_ishort_%i",i);
        sprintf(GAGG_ishort_name_histo,"GAGG_ishort_%i ; Channel ; Counts",i);
        GAGG_ishort[i] = new TH1F(GAGG_ishort_name, GAGG_ishort_name_histo, 4096, 0, 65536);
        }
    }
    cout << "OK Empty histograms created" << endl;

////////////////////////////////////////////////////////////
//             Declaring TTree and Variables              // 
//////////////////////////////////////////////////////////// 
    
// Connect tree file
    
    ifstream finp("inputlist.txt");
    string fileinput;
    TChain* tree = new TChain("EventTree");
   
    while(finp>>fileinput){
    tree -> Add(fileinput.c_str());
    cout<<"Input file: "<<fileinput<<endl;
    }
    
//Declaration of leaves types
    Int_t   MDPP16_ilong_Multiplicity;
    Int_t   MDPP16_ilong_Channels[16*3];
    Long64_t MDPP16_ilong_Values[16*3];
    Int_t   MDPP16_ishort_Multiplicity;
    Int_t   MDPP16_ishort_Channels[16*3];
    Long64_t MDPP16_ishort_Values[16*3];
    Int_t   MDPP16_TDC_time_diff_mul;
    Int_t   MDPP16_TDC_time_diff_Channels[16*3];
    Long64_t MDPP16_TDC_time_diff_Values[16*3];
    Int_t   MDPP16_Trigger_time_diff_mul;
    Int_t   MDPP16_Trigger_time_diff_Channels[16];
    Long64_t MDPP16_Trigger_time_diff_Values[16];
    Long64_t MDPP16_Extended_timestamps;
    Long64_t MDPP16_Event_timestamps;
   
    Int_t   Caen785_1_Multiplicity;
    Int_t   Caen785_1_Channels[32*3];
    Int_t   Caen785_1_Values[32*3];
    
    Int_t   Caen785_2_Multiplicity;
    Int_t   Caen785_2_Channels[32*3];
    Int_t   Caen785_2_Values[32*3];
    

   // Set branch addresses.
   tree->SetBranchAddress("MDPP16_ilong_Multiplicity",&MDPP16_ilong_Multiplicity);
   tree->SetBranchAddress("MDPP16_ilong_Channels",MDPP16_ilong_Channels);
   tree->SetBranchAddress("MDPP16_ilong_Values",MDPP16_ilong_Values);
   tree->SetBranchAddress("MDPP16_ishort_Multiplicity",&MDPP16_ishort_Multiplicity);
   tree->SetBranchAddress("MDPP16_ishort_Channels",MDPP16_ishort_Channels);
   tree->SetBranchAddress("MDPP16_ishort_Values",MDPP16_ishort_Values);
   tree->SetBranchAddress("MDPP16_TDC_time_diff_mul",&MDPP16_TDC_time_diff_mul);
   tree->SetBranchAddress("MDPP16_TDC_time_diff_Channels",MDPP16_TDC_time_diff_Channels);
   tree->SetBranchAddress("MDPP16_TDC_time_diff_Values",MDPP16_TDC_time_diff_Values);
   tree->SetBranchAddress("MDPP16_Trigger_time_diff_mul",&MDPP16_Trigger_time_diff_mul);
   tree->SetBranchAddress("MDPP16_Trigger_time_diff_Channels",MDPP16_Trigger_time_diff_Channels);
   tree->SetBranchAddress("MDPP16_Trigger_time_diff_Values",MDPP16_Trigger_time_diff_Values);
   tree->SetBranchAddress("MDPP16_Extended_timestamps",&MDPP16_Extended_timestamps);
   tree->SetBranchAddress("MDPP16_Event_timestamps",&MDPP16_Event_timestamps);
   
   tree->SetBranchAddress("Caen785_1_Multiplicity",&Caen785_1_Multiplicity);
   tree->SetBranchAddress("Caen785_1_Channels",Caen785_1_Channels);
   tree->SetBranchAddress("Caen785_1_Values",Caen785_1_Values);
   
   tree->SetBranchAddress("Caen785_2_Multiplicity",&Caen785_2_Multiplicity);
   tree->SetBranchAddress("Caen785_2_Channels",Caen785_2_Channels);
   tree->SetBranchAddress("Caen785_2_Values",Caen785_2_Values);
                           
////////////////////////////////////////////////////////////
//           Conditions histograms & variables            // 
//////////////////////////////////////////////////////////// 

// Study the multiplicity of our events on our detectors and modules
// Define our multiplicity histograms for DSSD
    TH1F *DSSD1_mul = new TH1F("DSSD1_multiplicity", "DSSD1 multiplicity ; Event multiplicity ; Counts",32*3, 0, 32*3);
// Define our multiplicity histograms for PAD
    TH1F *PADs_mul = new TH1F("PADs_multiplicity", "PADs multiplicity ; Event multiplicity ; Counts",32*3, 0, 32*3);
// Define our multiplicity histograms for GAGGs and CEPA
    TH1F *GAGG_ilong_mul  = new TH1F("GAGGs_and_CEPA_ilong_multiplicity",  "GAGGs and CEPA multiplicity ilong ; Event multiplicity ; Counts",16*3, 0, 16*3-1);
    TH1F *GAGG_ishort_mul = new TH1F("GAGGs_and_CEPA_ishort_multiplicity", "GAGGs and CEPA multiplicity ishort; Event multiplicity ; Counts",16*3, 0, 16*3-1);
// Hitmaps
    TH2F *hitmap_DSSD1 = new TH2F("hitmap_DSSD1", "hitmap_DSSD1 ; P ; N ; Hits", 16, 0, 16, 16, 0, 16);
// Bananas
    TH2F *BananaDSSD1_PAD1 = new TH2F("BananaDSSD1_PAD1", "Banana DSSD1 PAD1 ; Energy PAD1 ; Energy DSSD1 ; Counts", 200, 0, 4096, 200, 0, 4096);
// Ep vs En
    TH2F *Ep_vs_En_DSSD1 = new TH2F("Ep_vs_En_1", "E_{P} vs E_{N} ; E_{P} (keV) ; E_{N} (keV) ; Energy (keV)", 200, 0, 4096, 200, 0, 4096);
// Ep - En
    TH1F *Epn_DSSD1 = new TH1F("Epn_1", "E_{P} - E_{N} ; E_{P} - E_{N} (keV) ; Counts ; Energy (keV)", 500, -4096/2, 4096/2);

////////////////////////////////////////////////////////////
//                  LOOP OVER OUR TTREE                   // 
////////////////////////////////////////////////////////////    

// We are going to apply some conditions to our data and fill our histograms
    Long64_t nentries0 = tree->GetEntries();
    cout<< "Filling DSSD, CEPA and 2D histograms" << endl;
    cout<< " Total events: "<< nentries0/1e6 << " millions"<<endl;
    Long64_t totalEvents=0;
    
    Int_t DSSD1V_strip_triggered;
    Int_t DSSD1H_strip_triggered;
    Int_t GAGG_ilong_det_triggered, GAGG_ishort_det_triggered;
    Int_t PAD_det_triggered;
    
    Double_t Edssd1_p, Edssd1_n;
    Double_t Epad1;
    Double_t Egagg_ilong, Egagg_ishort;
    
//     Double_t EnergyDSSD1_V[16], EnergyDSSD1_H[16];
//     Double_t EnergyGAGG_ilong[4], EnergyGAGG_ishort[4];
//     Double_t EnergyPAD[1];
    
    Double_t DSSD1_PNdiff;
    Double_t DSSD1_PNmean;
    
    Int_t DSSD1_trigger;
    Int_t PAD1_trigger;
    Int_t GAGG_ilong_trigger, GAGG_ishort_trigger;
    
    Int_t DSSD1_goodEvent;
    
    Long64_t nbytes0 = 0;
    // Start of Ttree loop
    for (Long64_t j=0; j<nentries0;j++) {
      
        nbytes0 += tree->GetEntry(j);
        
        Edssd1_p=0; Edssd1_n=0; Epad1 = 0;
        Egagg_ilong=0; Egagg_ishort=0;
        
        DSSD1V_strip_triggered=0;  
        DSSD1H_strip_triggered=0;
        GAGG_ilong_det_triggered=0; GAGG_ishort_det_triggered=0;
        PAD_det_triggered=0;
        
        DSSD1_PNdiff=0;
        DSSD1_PNmean=0;
        
        DSSD1_trigger=0;
        PAD1_trigger=0;
        GAGG_ilong_trigger=0; GAGG_ishort_trigger=0;
        
        DSSD1_goodEvent=0;
        
//         for (Long64_t k=0; k<16; k++){ // initializing to 0 for each event
//             if (k<1){
//               EnergyPAD[k]=0;
//             }
//             if (k<4){
//               EnergyGAGG_ilong[k]=0;
//               EnergyGAGG_ishort[k]=0;
//             }
//             
//             EnergyDSSD1_V[k]=0;
//             EnergyDSSD1_H[k]=0;
            
//         } // end for to initialize variables
        
        // Fill multiplicity histograms
        if(MDPP16_ilong_Multiplicity  != 0){GAGG_ilong_mul  -> Fill(MDPP16_ilong_Multiplicity);}
        if(MDPP16_ishort_Multiplicity != 0){GAGG_ishort_mul -> Fill(MDPP16_ishort_Multiplicity);}
        if(Caen785_1_Multiplicity != 0){DSSD1_mul -> Fill(Caen785_1_Multiplicity);}
        if(Caen785_2_Multiplicity != 0){PADs_mul -> Fill(Caen785_2_Multiplicity);}
        
        // Check which detectors trigger
        if(Caen785_1_Multiplicity != 0){DSSD1_trigger=1;}        
        if(Caen785_2_Multiplicity != 0){for (Int_t k=0; k<Caen785_2_Multiplicity; k++){if (Caen785_2_Channels[k] == 0){PAD1_trigger=1;}}}
        if(MDPP16_ilong_Multiplicity  != 0){for (Int_t k=0; k<MDPP16_ilong_Multiplicity  ; k++){if(MDPP16_ilong_Channels[k] <= 7 ){GAGG_ilong_trigger=1;}}}
        if(MDPP16_ishort_Multiplicity != 0){for (Int_t k=0; k<MDPP16_ishort_Multiplicity ; k++){if(MDPP16_ishort_Channels[k]<= 7 ){GAGG_ishort_trigger=1;}}}

        ///////////////
        // DETECTORS //
        ///////////////
        
        if (DSSD1_trigger == 1){ // Check if detector triggers
          
//           for (Long64_t j0=0; j0<Caen785_1_Multiplicity; j0++) {
//             if (Caen785_1_Channels[j0] == 0) {Edssd1_p = Caen785_1_Values[j0]; DSSD1_V[Caen785_1_Channels[j0]] -> Fill(Edssd1_p);
//               
//               cout << Caen785_1_Channels[j0] << " " << Edssd1_p << endl;
//             }
//           }
          
//           if (PAD1_trigger == 1){ // Coincidence condition
//           if (std::any_of(std::begin(Caen785_1_Channels), std::end(Caen785_1_Channels), [=](Int_t ij){return ij > 15;}) && std::any_of(std::begin(Caen785_1_Channels), std::end(Caen785_1_Channels), [=](Int_t ij){return ij < 16;})){ // check for multiplicity 2 if exist 1 hit at P-side and other one at N-side

                for (Long64_t j0=0; j0<Caen785_1_Multiplicity; j0++) {
                    if (Caen785_1_Channels[j0] < 16) { // p-side
                        Edssd1_p = Caen785_1_Values[j0];
                        DSSD1V_strip_triggered = Caen785_1_Channels[j0];
                        DSSD1_V[DSSD1V_strip_triggered] -> Fill(Edssd1_p);
                    }
                    else { // n-side
                        Edssd1_n = Caen785_1_Values[j0];
                        DSSD1H_strip_triggered = Caen785_1_Channels[j0]-16;
                        DSSD1_H[DSSD1H_strip_triggered] -> Fill(Edssd1_n);
                    }
                    Ep_vs_En_DSSD1 -> Fill(Edssd1_p, Edssd1_n);
                    Epn_DSSD1 -> Fill(Edssd1_p - Edssd1_n);
                    hitmap_DSSD1 -> Fill(DSSD1V_strip_triggered, DSSD1H_strip_triggered);
                }
//             }
//             }
        } // end DSSD1
        
        if (PAD1_trigger == 1 ){ // 
          for (Long64_t j4=0; j4<Caen785_2_Multiplicity; j4++) {
            if (Caen785_2_Channels[j4] == 0) {Epad1 = Caen785_2_Values[j4]; PAD[Caen785_2_Channels[j4]] -> Fill(Epad1);
            }
          }
        } // end PAD 1
        
        if (GAGG_ilong_trigger == 1){
//         if (PAD1_trigger == 0 && PAD2_trigger == 0 && PAD3_trigger == 0 && PAD4_trigger == 0){ // Coincidence condition
            for (Long64_t j7=0; j7<MDPP16_ilong_Multiplicity; j7++) {
                if (MDPP16_ilong_Channels[j7] == 4){Egagg_ilong = MDPP16_ilong_Values[j7]; GAGG_ilong[0] -> Fill(Egagg_ilong);}
                if (MDPP16_ilong_Channels[j7] == 5){Egagg_ilong = MDPP16_ilong_Values[j7]; GAGG_ilong[1] -> Fill(Egagg_ilong);}
                if (MDPP16_ilong_Channels[j7] == 6){Egagg_ilong = MDPP16_ilong_Values[j7]; GAGG_ilong[2] -> Fill(Egagg_ilong);}
                if (MDPP16_ilong_Channels[j7] == 7){Egagg_ilong = MDPP16_ilong_Values[j7]; GAGG_ilong[3] -> Fill(Egagg_ilong);}
            }
//         }
        } // end MDPP16_ilong
        
//         if (GAGG_ishort_trigger == 1){
// //         if (PAD1_trigger == 0 && PAD2_trigger == 0 && PAD3_trigger == 0 && PAD4_trigger == 0){ // Coincidence condition
//             for (Long64_t j8=0; j8<MDPP16_ishort_Multiplicity; j8++) {
//                 if (MDPP16_ishort_Channels[j8] <= 7){Egagg_ishort = MDPP16_ishort_Values[j8]; GAGG_ishort_det_triggered = MDPP16_ishort_Channels[j8]; GAGG_ishort[GAGG_ishort_det_triggered] -> Fill(Egagg_ishort);}
//             }
// //         }
//         } // end MDPP16_ishort
        
//         /////////////////////////
//         // ¬¬¬¬¬ BANANAS ¬¬¬¬¬ //
//         /////////////////////////
//         
        // Telescope 1
        if (DSSD1_trigger == 1 && PAD1_trigger == 1 /*&& DSSD1V_strip_triggered == 7*/) {
          BananaDSSD1_PAD1 -> Fill(Epad1, Edssd1_p);
        }
        
        if(totalEvents%1000000 == 0)cout<<totalEvents/1e6<<" millions read"<<endl;
        totalEvents ++;
        
    } // End LOOP OVER TTREE EVENTS

    cout<<"DSSD, SiBall and CEPA histograms filled"<<endl;

////////////////////////////////////////////////////////////
//                 Opening file to write                  // 
//////////////////////////////////////////////////////////// 
    
    cout << " Opening output file " << endl;
    TFile *output_file = new TFile("TEAFN_207Bi.root", "recreate");
        
    TDirectory *multiplicity = output_file->mkdir("Multiplicity");
    
    TDirectory *No_calib = output_file->mkdir("No_calibrated");
    TDirectory *DSSD1_dir = No_calib->mkdir("DSSD1");
    TDirectory *PADs_dir = No_calib->mkdir("PADs");
    TDirectory *GAGG_ilong_dir = No_calib->mkdir("GAGG_ilong");
    TDirectory *GAGG_ishort_dir = No_calib->mkdir("GAGG_ishort");
    TDirectory *Hitmaps = No_calib->mkdir("Hitmaps");
    TDirectory *Bananas = No_calib->mkdir("Bananas");
    TDirectory *Epn = No_calib->mkdir("Epn");
    TDirectory *P_vs_N = No_calib->mkdir("P_vs_N");
    
    cout << " Output file opened " << endl;
   
////////////////////////////////////////////////////////////
//                  Writing output file                   // 
////////////////////////////////////////////////////////////
   
   for (Long64_t k=0; k<32;k++) {
        if (k<1) {
            PADs_dir->cd();
            PAD[k]->Write();
        }
        if (k<4) {
            GAGG_ilong_dir->cd();
            GAGG_ilong[k]->Write();
        
            GAGG_ishort_dir->cd();
            GAGG_ishort[k]->Write();
        }
        if (k<16) {
            
            DSSD1_dir->cd();
            DSSD1_V[k]->Write();
        }
        else {
            
            DSSD1_dir->cd();
            DSSD1_H[k-16]->Write();
        }
    }
    
// Multiplicity folder
    multiplicity -> cd();
    DSSD1_mul -> Write();
    PADs_mul  -> Write();
    GAGG_ilong_mul -> Write();
    GAGG_ishort_mul -> Write();
    
// Bananas
    Bananas -> cd();
    BananaDSSD1_PAD1 -> Write();
    
// Hitmaps
    Hitmaps -> cd();
    hitmap_DSSD1 -> Write();
    
// P - N
    Epn -> cd();
    Epn_DSSD1 -> Write();
    
// P vs N
    P_vs_N -> cd();
    Ep_vs_En_DSSD1 -> Write();
    
   output_file -> Close();
   cout<<"All histograms saved in .root file"<<endl;
   
   cout << "Program finished at:  " << endl; 
   system("/bin/date"); 
   cout << endl;
   
   return 0;
}

void analyze(){
    main();
}
