#include <stdio.h>
#include <string.h>
#include <iostream>     // std::cout, std::end
#include <fstream>

#include "TFile.h"
#include "TTree.h"
#include "TH1D.h"

////////////////////////////
// Initializing variables //
////////////////////////////

using std::ifstream;

// To compile:
// g++ mvme_unpacker.cpp `root-config --cflags --libs` -o mvme_unpacker.out
// To execute:
// ./mvme_unpacker.out inputfile.mvlclst outputfile.root

// If you want to customize this program with a different number of modules you should add new variables, definitions, initialization
// and add your modules with same order they appear in MVMe software in readFrame function. 
// If you want to add different modules than Caen785, MDPP-16 or Caen1190A you have to create new functions to read those modules.

using namespace std;

//////////////////////////////////////////////////
// All variables initialization, event and tree //
//////////////////////////////////////////////////
// Checking
ULong_t bytes_counter = 0;
ULong_t totalEvents = 0;
Int_t wrong_module_numbers = 0;
Int_t continue_frame_counter = 0;
Int_t broken_event_count = 0;
Int_t EOF_MARKER = 0;
Int_t DATAFRAME_marker = 0;

// Variables that will be determined by the different modules that we have. 
// We use it as TTree branches.

// Caen785 ADC 1
Int_t Caen785_1_Multiplicity;
Int_t Caen785_1_Channel[32*3];
Int_t Caen785_1_Value[32*3];

TH1F *Caen785_1_histo[32];
char Caen785_1_name[64], Caen785_1_histo_name[64];

// Caen785 ADC 2
Int_t Caen785_2_Multiplicity;
Int_t Caen785_2_Channel[32*3];
Int_t Caen785_2_Value[32*3];

TH1F *Caen785_2_histo[32];
char Caen785_2_name[64], Caen785_2_histo_name[64];

// Caen785 ADC 3
Int_t Caen785_3_Multiplicity;
Int_t Caen785_3_Channel[32*3];
Int_t Caen785_3_Value[32*3];

TH1F *Caen785_3_histo[32];
char Caen785_3_name[64], Caen785_3_histo_name[64];

// Caen1190A TDC
Int_t Caen1190a_Multiplicity;
Int_t Caen1190a_Channel[128];
Int_t Caen1190a_Value[128];

// MDPP16 QDC
// this module have 16 channels but data follows this scheme:
//   Integration long        - channels [0-15]
//   Integration short       - channels [48-63]
//   TDC time difference     - channels [16-31]
//   Trigger time difference - channels [32(T0),33(T1)]

Int_t   MDPP16_ilong_Multiplicity;
Int_t   MDPP16_ilong_Channel[16*3];
ULong_t MDPP16_ilong_Value[16*3];
Int_t   MDPP16_ishort_Multiplicity;
Int_t   MDPP16_ishort_Channel[16*3];
ULong_t MDPP16_ishort_Value[16*3];
Int_t   MDPP16_TDC_time_diff_mul;
Int_t   MDPP16_TDC_time_diff_Channel[16*3];
ULong_t MDPP16_TDC_time_diff_Value[16*3];
Int_t   MDPP16_Trigger_time_diff_mul;
Int_t   MDPP16_Trigger_time_diff_Channel[16];
ULong_t MDPP16_Trigger_time_diff_Value[16];
ULong_t MDPP16_Extended_timestamp;
ULong_t MDPP16_Event_timestamp;

TH1F *MDPP16_ilong_histo[16], *MDPP16_ishort_histo[16];
char MDPP16_ilong_name[64], MDPP16_ilong_histo_name[64];
char MDPP16_ishort_name[64], MDPP16_ishort_histo_name[64];

// MDPP32 SCP 1
// this module have 32 channels but data follows this scheme:
//   ADC Value               - channels [0-31]
//   TDC time difference     - channels [32-63]
//   Trigger time difference - channels [64(T0),65(T1)]
Int_t   MDPP32_1_Multiplicity;
Int_t   MDPP32_1_Channel[32*3];
ULong_t MDPP32_1_Value[32*3];
Int_t   MDPP32_1_TDC_time_diff_mul;
Int_t   MDPP32_1_TDC_time_diff_Channel[32*3];
ULong_t MDPP32_1_TDC_time_diff_Value[32*3];
Int_t   MDPP32_1_Trigger_time_diff_mul;
Int_t   MDPP32_1_Trigger_time_diff_Channel[32];
ULong_t MDPP32_1_Trigger_time_diff_Value[32];
ULong_t MDPP32_1_Extended_timestamp;
ULong_t MDPP32_1_Event_timestamp;

TH1F *MDPP32_1_histo[32];
char MDPP32_1_name[64], MDPP32_1_histo_name[64];

// MDPP32 SCP 2
Int_t   MDPP32_2_Multiplicity;
Int_t   MDPP32_2_Channel[32*3];
ULong_t MDPP32_2_Value[32*3];
Int_t   MDPP32_2_TDC_time_diff_mul;
Int_t   MDPP32_2_TDC_time_diff_Channel[32*3];
ULong_t MDPP32_2_TDC_time_diff_Value[32*3];
Int_t   MDPP32_2_Trigger_time_diff_mul;
Int_t   MDPP32_2_Trigger_time_diff_Channel[32];
ULong_t MDPP32_2_Trigger_time_diff_Value[32];
ULong_t MDPP32_2_Extended_timestamp;
ULong_t MDPP32_2_Event_timestamp;

TH1F *MDPP32_2_histo[32];
char MDPP32_2_name[64], MDPP32_2_histo_name[64];

// MDPP32 SCP 3
Int_t   MDPP32_3_Multiplicity;
Int_t   MDPP32_3_Channel[32*3];
ULong_t MDPP32_3_Value[32*3];
Int_t   MDPP32_3_TDC_time_diff_mul;
Int_t   MDPP32_3_TDC_time_diff_Channel[32*3];
ULong_t MDPP32_3_TDC_time_diff_Value[32*3];
Int_t   MDPP32_3_Trigger_time_diff_mul;
Int_t   MDPP32_3_Trigger_time_diff_Channel[32];
ULong_t MDPP32_3_Trigger_time_diff_Value[32];
ULong_t MDPP32_3_Extended_timestamp;
ULong_t MDPP32_3_Event_timestamp;

TH1F *MDPP32_3_histo[32];
char MDPP32_3_name[64], MDPP32_3_histo_name[64];

// MDPP32 SCP 4
Int_t   MDPP32_4_Multiplicity;
Int_t   MDPP32_4_Channel[32*3];
ULong_t MDPP32_4_Value[32*3];
Int_t   MDPP32_4_TDC_time_diff_mul;
Int_t   MDPP32_4_TDC_time_diff_Channel[32*3];
ULong_t MDPP32_4_TDC_time_diff_Value[32*3];
Int_t   MDPP32_4_Trigger_time_diff_mul;
Int_t   MDPP32_4_Trigger_time_diff_Channel[32];
ULong_t MDPP32_4_Trigger_time_diff_Value[32];
ULong_t MDPP32_4_Extended_timestamp;
ULong_t MDPP32_4_Event_timestamp;

TH1F *MDPP32_4_histo[32];
char MDPP32_4_name[64], MDPP32_4_histo_name[64];

// MADC32 1
Int_t   MADC32_1_Multiplicity;
Int_t   MADC32_1_Channel[32*3];
ULong_t MADC32_1_Value[32*3];
ULong_t MADC32_1_Extended_timestamp;
ULong_t MADC32_1_Event_timestamp;

TH1F *MADC32_1_histo[32];
char MADC32_1_name[64], MADC32_1_histo_name[64];

// MADC32 2
Int_t   MADC32_2_Multiplicity;
Int_t   MADC32_2_Channel[32*3];
ULong_t MADC32_2_Value[32*3];
ULong_t MADC32_2_Extended_timestamp;
ULong_t MADC32_2_Event_timestamp;

TH1F *MADC32_2_histo[32];
char MADC32_2_name[64], MADC32_2_histo_name[64];

// VMMR-8
ULong_t VMMR8_Multiplicity;
ULong_t VMMR8_Channel[1024*3];
ULong_t VMMR_Value[1024*3];
Int_t   VMMR8_time_diff_mul;
Int_t   VMMR8_Time_diff_bus[16*3];
ULong_t VMMR8_Time_diff_value[16*3];
ULong_t VMMR8_Extended_timestamp;
ULong_t VMMR8_Event_timestamp;

TH1F *VMMR8_histo[1024];
char VMMR8_name[64], VMMR8_histo_name[64];

//Histogram LOOP
void histoLOOP(){
  for (Int_t i=0; i<1024; i++) {
    if (i<16) {
      sprintf(MDPP16_ilong_name,"MDPP16_ilong_%i",i);
      sprintf(MDPP16_ilong_histo_name,"MDPP16_ilong_%i ; Channel ; Counts",i);
      MDPP16_ilong_histo[i] = new TH1F(MDPP16_ilong_name, MDPP16_ilong_histo_name, 4096, 0, 65536);
            
      sprintf(MDPP16_ishort_name,"MDPP16_ishort_%i",i);
      sprintf(MDPP16_ishort_histo_name,"MDPP16_ishort_%i ; Channel ; Counts",i);
      MDPP16_ishort_histo[i] = new TH1F(MDPP16_ishort_name, MDPP16_ishort_histo_name, 4096, 0, 65536);
    }
    if (i<32) {
      sprintf(Caen785_1_name,"Caen785_1_%i",i);
      sprintf(Caen785_1_histo_name,"Caen785_1_%i ; Channel ; Counts",i);
      Caen785_1_histo[i] = new TH1F(Caen785_1_name, Caen785_1_histo_name, 4096, 0, 4096);
      
      sprintf(Caen785_2_name,"Caen785_2_%i",i);
      sprintf(Caen785_2_histo_name,"Caen785_2_%i ; Channel ; Counts",i);
      Caen785_2_histo[i] = new TH1F(Caen785_2_name, Caen785_2_histo_name, 4096, 0, 4096);
    }
  }
}

// Very large buffer as f->read needs a defined variable to storage what reads. Only used in jumpFrame and jumpModule.
char discard[4 * 65536];

// TFile
TFile *treeFile;

// TTree
TTree *EventTree; 

// initializes all data before an event  
void initEvent() 
{
  // Module 1
  MDPP16_ilong_Multiplicity = 0;       
  MDPP16_ishort_Multiplicity = 0;
  MDPP16_TDC_time_diff_mul = 0;        
  MDPP16_Trigger_time_diff_mul = 0;    
  MDPP16_Extended_timestamp = 0;       
  MDPP16_Event_timestamp = 0;   
  
  // Module 1                           // Module 2
  Caen785_1_Multiplicity = 0;           Caen785_2_Multiplicity = 0;
  
  for (int i=0; i<1024*3; i++) {
    
    if (i<16*3){
      // Module 1
      MDPP16_ilong_Channel[i] = 0;               
      MDPP16_ilong_Value[i] = 0;
      MDPP16_ishort_Channel[i] = 0;              
      MDPP16_ishort_Value[i] = 0;
      MDPP16_TDC_time_diff_Channel[i] = 0;       
      MDPP16_TDC_time_diff_Value[i] = 0;
    }
    if (i<32*3){
      // Module 1                         // Module 2               
      Caen785_1_Channel[i] = 0;            Caen785_2_Channel[i] = 0;
      Caen785_1_Value[i] = 0;              Caen785_2_Value[i] = 0;

    }
  }
}

// Generate TTree structure
void createTree() 
{
  // MDPP16_QDC 
  EventTree->Branch("MDPP16_ilong_Multiplicity",             &MDPP16_ilong_Multiplicity,            "MDPP16_ilong_Multiplicity/I");
  EventTree->Branch("MDPP16_ilong_Channels",                  MDPP16_ilong_Channel,                 "MDPP16_ilong_Channels[MDPP16_ilong_Multiplicity]/I");
  EventTree->Branch("MDPP16_ilong_Values",                    MDPP16_ilong_Value,                   "MDPP16_ilong_Values[MDPP16_ilong_Multiplicity]/L");
                                                                                                    
  EventTree->Branch("MDPP16_ishort_Multiplicity",            &MDPP16_ishort_Multiplicity,           "MDPP16_ishort_Multiplicity/I");
  EventTree->Branch("MDPP16_ishort_Channels",                 MDPP16_ishort_Channel,                "MDPP16_ishort_Channels[MDPP16_ishort_Multiplicity]/I");
  EventTree->Branch("MDPP16_ishort_Values",                   MDPP16_ishort_Value,                  "MDPP16_ishort_Values[MDPP16_ishort_Multiplicity]/L");
                                                                                                    
  EventTree->Branch("MDPP16_TDC_time_diff_mul",              &MDPP16_TDC_time_diff_mul,             "MDPP16_TDC_time_diff_mul/I");
  EventTree->Branch("MDPP16_TDC_time_diff_Channels",          MDPP16_TDC_time_diff_Channel,         "MDPP16_TDC_time_diff_Channels[MDPP16_TDC_time_diff_mul]/I");
  EventTree->Branch("MDPP16_TDC_time_diff_Values",            MDPP16_TDC_time_diff_Value,           "MDPP16_TDC_time_diff_Values[MDPP16_TDC_time_diff_mul]/L");
                                                                                                    
  EventTree->Branch("MDPP16_Trigger_time_diff_mul",          &MDPP16_Trigger_time_diff_mul,         "MDPP16_Trigger_time_diff_mul/I");
  EventTree->Branch("MDPP16_Trigger_time_diff_Channels",      MDPP16_Trigger_time_diff_Channel,     "MDPP16_Trigger_time_diff_Channels[MDPP16_Trigger_time_diff_mul]/I");
  EventTree->Branch("MDPP16_Trigger_time_diff_Values",        MDPP16_Trigger_time_diff_Value,       "MDPP16_Trigger_time_diff_Values[MDPP16_Trigger_time_diff_mul]/L");
                                                                                                    
  EventTree->Branch("MDPP16_Extended_timestamps",            &MDPP16_Extended_timestamp,            "MDPP16_Extended_timestamps/L");
  EventTree->Branch("MDPP16_Event_timestamps",               &MDPP16_Event_timestamp,               "MDPP16_Event_timestamps/L");
                                                                                                    
  // Caen785 1                                                                                   
  EventTree->Branch("Caen785_1_Multiplicity",                &Caen785_1_Multiplicity,                "Caen785_1_Multiplicity/I");
  EventTree->Branch("Caen785_1_Channels",                     Caen785_1_Channel,                     "Caen785_1_Channels[Caen785_1_Multiplicity]/I");
  EventTree->Branch("Caen785_1_Values",                       Caen785_1_Value,                       "Caen785_1_Values[Caen785_1_Multiplicity]/I");
  
  // Caen785 2                                                                                   
  EventTree->Branch("Caen785_2_Multiplicity",                &Caen785_2_Multiplicity,                "Caen785_2_Multiplicity/I");
  EventTree->Branch("Caen785_2_Channels",                     Caen785_2_Channel,                     "Caen785_2_Channels[Caen785_2_Multiplicity]/I");
  EventTree->Branch("Caen785_2_Values",                       Caen785_2_Value,                       "Caen785_2_Values[Caen785_2_Multiplicity]/I");
}

///////////////////////
// Functions to jump //
///////////////////////
// There are some types of frames or some modules that we want to jump over

void jumpFrame(ifstream *f, unsigned char frame_header[4]) 
{
//     cout  << to_string(frame_header[0]) << " " << to_string(frame_header[1]) << " " << to_string(frame_header[2]) << " " << to_string(frame_header[3]) << endl;
    
  int words_length = (frame_header[0] + (frame_header[1] << 8) ) & 0b0001111111111111;
//   cout << "Words to jump " << words_length << endl;
  
  if (words_length != 0) {
    f->read(discard, words_length * 4);
  }
  bytes_counter += (words_length * 4);
}

void jumpModule(ifstream *f)
{
  unsigned char block_read_header[4];
  f->read((char*) block_read_header, 4);
    
  unsigned short words_length = ( block_read_header[0] +  
                                (block_read_header[1] << 8) ) & 0b0001111111111111;
    
  if (words_length != 0) {
    f->read(discard, words_length * 4);
  }
}

///////////////////////////////
// Functions to read Caen785 //
///////////////////////////////
// Input1: File that we are reading
// Input2: Module number (should be 1 by default)

void readCAEN785data(ifstream *f, int module)
{
  unsigned char caen785_data[4];
  f->read((char*) caen785_data, 4);
//   unsigned short data_check = (caen785_data[2] >> 5) & 0b00000111111;
  unsigned short data_check = ((caen785_data[2] + (caen785_data[3] << 8)) >> 5) & 0b00000111111;
    
  if (data_check == 0b000000) {
    unsigned char channel = caen785_data[2] & 0b00011111;
    unsigned int value = (caen785_data[0] + (caen785_data[1] << 8))  & 0b0000111111111111;

    if (module == 1) {
      Caen785_1_Channel[Caen785_1_Multiplicity] = channel;
      Caen785_1_Value[Caen785_1_Multiplicity] = value;
      Caen785_1_histo[channel] -> Fill(value);
      Caen785_1_Multiplicity++;
        if (Caen785_1_Multiplicity > 32*3-1) { Caen785_1_Multiplicity = 32*3-1; }
    }
        
    if (module == 2){
      Caen785_2_Channel[Caen785_2_Multiplicity] = channel;
      Caen785_2_Value[Caen785_2_Multiplicity] = value;
      Caen785_2_histo[channel] -> Fill(value);
      Caen785_2_Multiplicity++;
      if (Caen785_2_Multiplicity > 32*3-1) { Caen785_2_Multiplicity = 32*3-1; }
    }
      
    if (module == 3){
      Caen785_3_Channel[Caen785_3_Multiplicity] = channel;
      Caen785_3_Value[Caen785_3_Multiplicity] = value;
      Caen785_3_histo[channel] -> Fill(value);
      Caen785_3_Multiplicity++;
      if (Caen785_3_Multiplicity > 32*3-1) { Caen785_3_Multiplicity = 32*3-1; }
    }
  }
  // if the word isnt a data, it will read but wont do anything
}

void readCAEN785(ifstream *f, int module) 
{
  unsigned char block_read_header[4];
  f->read((char*) block_read_header, 4); // should be Type = 0xf5

  unsigned short module_event_length = (block_read_header[0] + (block_read_header[1] << 8) ) & 0b0001111111111111;
    
  if (module_event_length > 1) {
    unsigned char module_header[4];
    f->read((char*) module_header, 4);
        
    for (int i=0; i < module_event_length - 1; i++ ) {
        readCAEN785data(f, module);
    }
    }
    else if (module_event_length == 1) { 
      //Some events have length 1, for unknown reasons
      unsigned char dontcare[4];
      f->read((char*) dontcare, 4);
      broken_event_count++;
    }
}

//////////////////////////////////
// Functions to read MDPP16-QDC //
//////////////////////////////////
// Reading each 32-bit data words, if the number of channels that have a count 
// is odd,  mvme adds a 00 00 00 00 word before End of Event.

void readMDPP16_QDC_data(ifstream *f)
{
  unsigned char mdpp16_qdc_data[4];
  f->read((char*) mdpp16_qdc_data, 4);
    
  unsigned char mdpp16_qdc_data_check = (mdpp16_qdc_data[3] >> 4) & 0b1111;
    
  if (mdpp16_qdc_data_check == 0b0001) {
        
    if (((mdpp16_qdc_data[0] + (mdpp16_qdc_data[1] << 8) + (mdpp16_qdc_data[2] << 16) + (mdpp16_qdc_data[3] << 24)) != 0)) {
            
      unsigned int channel = mdpp16_qdc_data[2] & 0b00111111;
      unsigned long value = ( mdpp16_qdc_data[0] + (mdpp16_qdc_data[1] << 8) ) & 0b1111111111111111;
            
      if ((channel >= 0) && (channel < 16)){
        MDPP16_ilong_Channel[MDPP16_ilong_Multiplicity] = channel;
        MDPP16_ilong_Value[MDPP16_ilong_Multiplicity] = value;
        MDPP16_ilong_Multiplicity++;
        MDPP16_ilong_histo[channel] -> Fill(value);
        if (MDPP16_ilong_Multiplicity > 16*3-1) { MDPP16_ilong_Multiplicity = 16*3-1; }
      }
            
      if ((channel >= 48) && (channel < 64)){
        MDPP16_ishort_Channel[MDPP16_ishort_Multiplicity] = channel-48;
        MDPP16_ishort_Value[MDPP16_ishort_Multiplicity] = value;
        MDPP16_ishort_Multiplicity++;
        MDPP16_ishort_histo[channel-48] -> Fill(value);
        if (MDPP16_ishort_Multiplicity > 16*3-1) { MDPP16_ishort_Multiplicity = 16*3-1; }
      }
      
      if ((channel >= 16) && (channel < 32)){
        MDPP16_TDC_time_diff_Channel[MDPP16_TDC_time_diff_mul] = channel - 16;
        MDPP16_TDC_time_diff_Value[MDPP16_TDC_time_diff_mul] = value;
        MDPP16_TDC_time_diff_mul++;
        if(MDPP16_TDC_time_diff_mul > 16*3-1){MDPP16_TDC_time_diff_mul = 16*3-1;}
      }
      
      if((channel >= 32) && (channel < 34)){
        MDPP16_Trigger_time_diff_Channel[MDPP16_Trigger_time_diff_mul] = channel-32;
        MDPP16_Trigger_time_diff_Value[MDPP16_Trigger_time_diff_mul] = value;
        MDPP16_Trigger_time_diff_mul++;
        if(MDPP16_Trigger_time_diff_mul > 16-1){MDPP16_Trigger_time_diff_mul = 16-1;}
      }
    } // != 0
  }
  
  if(mdpp16_qdc_data_check == 0b0010){
    MDPP16_Extended_timestamp = (mdpp16_qdc_data[0] + (mdpp16_qdc_data[1] << 8) ) & 0b1111111111111111;
  }
  
  if (((mdpp16_qdc_data[3] >> 6) & 0b11) == 3){ // Reading timestamp in the end of event
    MDPP16_Event_timestamp = (mdpp16_qdc_data[0] + (mdpp16_qdc_data[1] << 8) + (mdpp16_qdc_data[2] << 16) + ((mdpp16_qdc_data[3] & 0b00111111) << 24));
  }
}

void readMDPP16_QDC(ifstream *f) 
{
  
  unsigned char block_read_header[4];
  f->read((char*) block_read_header, 4); // should be Type = 0xf5
  unsigned short module_event_length = (block_read_header[0] + (block_read_header[1] << 8) ) & 0b0001111111111111;
    
  if (module_event_length > 1) {
    unsigned char module_header[4];
    f->read((char*) module_header, 4);
    for (int i=0; i < module_event_length-1; i++) {
      readMDPP16_QDC_data(f);
    }        
  } else if (module_event_length == 1) {
      unsigned char dontCare[4];
      f->read((char*) dontCare, 4);
      broken_event_count++;
    }
//     cout << "reading MDPP16: " << module_event_length << " values" << endl; 
}   

//////////////////////////////////////
// Functions to read each dataFrame //
//////////////////////////////////////
// reads a frame, that usually contains an event  

void readFrame(ifstream *f) 
{
    unsigned char header[4];
    f->read((char*) header, 4);
    unsigned char frameType = header[3];
    bytes_counter += 4;
    DATAFRAME_marker = 0;
//     cout << "HEADER: "<< std::hex << (int) header[0] << " " << (int) header[1] << " " << (int) header[2] << " " << (int) header[3] << " " << endl;
    
    
    if (frameType == 0xFA) { 
//         cout << " Frame Type is: " << std::hex << (int) frameType << endl;
        // Those frames don't have relevant info, but EOF_MARKER, so we jump it always.
        
        unsigned int frame_subtype;
        frame_subtype = (header[1] + (header[2] << 8)) >> 5 & 0b1111111;
//         cout << " Frame sub-Type is: " << std::hex << (int) frame_subtype << endl;
        
        if (frame_subtype == 0x77) { // EOF_MARKER subtype
            cout <<  "EOF_MARKER " << header << endl;
            EOF_MARKER = 1;
        } 
        else {
            jumpFrame(f, header);
        }
    }
    
    else if (frameType == 0xF3) {
        DATAFRAME_marker = 1;
        // This frame marks the start of an event, so keep reading. Always followed by 0xF5 frame. 
        // Sometimes if data contained in this frame are too large, 0xF3 frames split in some more, so 
        // we have to check the Continue bit. If Continue bit is set, the following frames will be 
        // StackContinuation Frames (0xF9). The last frame in the series will have Continue bit cleared.
      
//       if (totalEvents > 1485980) {
//             cout << "Event " << totalEvents << " Bytes counted " << bytes_counter << endl;
//             cout << "HEADER: "<< std::hex << (int) header[0] << " " << (int) header[1] << " " << (int) header[2] << " " << (int) header[3] << " " << endl;
//         }
      
        initEvent();
        jumpModule(f);
        readMDPP16_QDC(f);
//         jumpModule(f);
        readCAEN785(f,1);
//         jumpModule(f);
        readCAEN785(f,2);
//         jumpModule(f);
        
        
        if (totalEvents % 1000000 == 0) {
            cout << " Events counted: " << totalEvents/1e6 << " millions" << endl;
        }
        
        if (header[2] >> 7 == 0b1) 
        {
            //cout << "continue frame" << endl;
            continue_frame_counter += 1;
        }
        
        totalEvents++;
        
        int frame_F3_length = (header[0] + (header[1] << 8) ) & 0b0001111111111111;
        bytes_counter += frame_F3_length * 4;
        
    }
    
    else if (!frameType){
      cout << "-- No EOF found so stop reading the file--" << endl;
      EOF_MARKER = 1;
    }
    
    else {
        // cout << " ERROR: unknown frame type: '" << (int) frameType << "'" << endl;
        // cout << "jumping frame" << endl;
        jumpFrame(f, header);
    }
}
 
///////////////////////////////////////////////
// Functions to read files and generate tree //
///////////////////////////////////////////////
// Main function. Opens the file and proceeds according 
// with the command-line parameters
// filename[1] must be input file name and filename[2] output file name

int main (int len, char* filename[]) 
{      
    ifstream fp;
    fp.open(filename[1], ios::in | ios::binary);
    printf("%s\n", filename[1]);
    
    char dontcare[8];
    fp.read(dontcare , 8);    // Jump over MVLC_USB ascii and littleE marker
    bytes_counter += 8;
    
    EventTree = new TTree("EventTree", "EventTree");
    treeFile = new TFile(filename[2], "recreate");    
    createTree();
    histoLOOP();

    while (EOF_MARKER == 0) {
//     while(!fp.eof()){
//     for (int w=0; w < 8; w++) {
        readFrame(&fp);
        if(DATAFRAME_marker == 1){EventTree->Fill();}
        if(fp.eof()){EOF_MARKER=1;}
//         cout << "Bytes counted: " << bytes_counter << endl;
    }
        
    cout << "File read, EOF_MARKER checked, total events: " << totalEvents << endl;
//     cout << "Events with unexpected module numbers: " << wrong_module_numbers << endl;    
//     cout << "Events with continue frames: " << continue_frame_counter << endl;
//     cout << "Broken events at caen (sum of all): " <<  broken_event_count << endl;

    //save the Tree header. The file will be automatically closed
    //when going out of the function scope
    EventTree->Write();
    
    TDirectory *MDPP16_ilong_dir = treeFile->mkdir("MDPP16_ilong");
    TDirectory *MDPP16_ishort_dir = treeFile->mkdir("MDPP16_ishort");
    TDirectory *Caen785_1_dir = treeFile->mkdir("Caen785_1");
    TDirectory *Caen785_2_dir = treeFile->mkdir("Caen785_2");
    
    
    for (int i=0; i<1024; i++){
      if (i<16){
        MDPP16_ilong_dir->cd();
        MDPP16_ilong_histo[i] -> Write();
        
        MDPP16_ishort_dir->cd();
        MDPP16_ishort_histo[i] -> Write();
      }
      
      if (i<32){
        Caen785_1_dir->cd();
        Caen785_1_histo[i] -> Write();
          
        Caen785_2_dir->cd();
        Caen785_2_histo[i] -> Write();
      }
    }
        
    cout << "done." << endl;
    treeFile->Close();
    fp.close();   
    
}
