///////////////////////////////////////////////////////////////////////////////
//           ANALYSIS PROGRAM FOR TEAFN He+LaBr SETUP                        // 
//                                            Vicente García Távora Sept 2022//
///////////////////////////////////////////////////////////////////////////////


//C,C++ Libraries
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <cstdlib>
#include <fstream>
#include <algorithm> // for std::find
#include <iterator> // for std::begin, std::end
// ROOT libraries
#include <TROOT.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TTree.h>
#include <TFile.h>
#include <TF1.h>
#include <TRandom3.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TColor.h>
#include <TStyle.h>
#include <TCutG.h>
using namespace std;

Int_t main(){

    cout << "Program started at:  " << endl; 
    system("/bin/date"); 
    cout << endl;
    
////////////////////////////////////////////////////////////
//            Read calibration parameters file            // 
////////////////////////////////////////////////////////////
// read calibration parameter file and store values in bidimensional arrays
// ={}{} fill all with 0
    
//     ifstream file("calibrations/multifit/calibration_parameters.txt");
//     
//      Double_t mval, nval;
//      Int_t adc, ch_adc;
//      Double_t m[2][1]; Double_t n[2][1];
//      
//      for (Int_t j=0; j<2; j++){
//          for(Int_t i=0; i<1; i++){
//              m[j][i]=0; n[j][i]=0;
//          }
//      }
//     
//     while (file >> mval >> nval >> adc >> ch_adc){
//      
//      m[adc][ch_adc] = mval;
//      n[adc][ch_adc] = nval;
//     }
//     file.close();
    
// with index as m[j][i] 
// Det=0  -> LaBr
// Det1=1 -> HPGe
    
////////////////////////////////////////////////////////////
//               Empty histograms generator               // 
////////////////////////////////////////////////////////////

    TH1F *Scint[2];
    TH1F *ScintCalib[2];
    char MADC32_name[60], MADC32_name_histo[60];
// Loop to create empty histograms
    cout << "Creating empty histograms" << endl;
    for (Int_t i=0; i<2;i++) {
        sprintf(MADC32_name,"MADC32_%i",i);
        sprintf(MADC32_name_histo,"MADC32_%i ; Channel ; Counts",i);
        Scint[i] = new TH1F(MADC32_name, MADC32_name_histo, 4096*2, 0, 4096*2);
        
        //ScintCalib[i] = new TH1F(MADC32_name, MADC32_name_histo, 4096*2, n[i][0], m[i][0]*4096*2+n[i][0]);
        }
    cout << "OK Empty histograms created" << endl;

////////////////////////////////////////////////////////////
//             Declaring TTree and Variables              // 
//////////////////////////////////////////////////////////// 
    
// Connect tree file
    
    ifstream finp("inputlist.txt");
    string fileinput;
    TChain* tree = new TChain("EventTree");
   
    while(finp>>fileinput){
    tree -> Add(fileinput.c_str());
    cout<<"Input file: "<<fileinput<<endl;
    }
    
//Declaration of leaves types
    Int_t   MADC32_1_Multiplicity;
    Int_t   MADC32_1_Channels[32*3];
    Long64_t MADC32_1_Values[32*3];
    Long64_t MADC32_1_Extended_timestamps;
    Long64_t MADC32_1_Event_timestamps;

   // Set branch addresses.
   tree->SetBranchAddress("MADC32_1_Multiplicity",&MADC32_1_Multiplicity);
   tree->SetBranchAddress("MADC32_1_Channels",MADC32_1_Channels);
   tree->SetBranchAddress("MADC32_1_Values",MADC32_1_Values);
   tree->SetBranchAddress("MADC32_1_Extended_timestamps",&MADC32_1_Extended_timestamps);
   tree->SetBranchAddress("MADC32_1_Event_timestamps",&MADC32_1_Event_timestamps);
                           
////////////////////////////////////////////////////////////
//           Conditions histograms & variables            // 
//////////////////////////////////////////////////////////// 

// Define our multiplicity histograms for Scint and HPGe
    TH1F *MADC32_mul  = new TH1F("Scint_multiplicity",  "HPGe and Pyramid multiplicity ; Event multiplicity ; Counts",32*3, 0, 32*3);

////////////////////////////////////////////////////////////
//                  LOOP OVER OUR TTREE                   // 
////////////////////////////////////////////////////////////    

// We are going to apply some conditions to our data and fill our histograms
    Long64_t nentries0 = tree->GetEntries();
    cout<< "Filling HPGe, Pyramid and 2D histograms" << endl;
    cout<< " Total events: "<< nentries0/1e6 << " millions"<<endl;
    Long64_t totalEvents=0;
    
    Int_t HPGe_trigger, Pyramid_trigger;
    
    Double_t E_HPGe, E_pyramid;
    
    Double_t Energy_MADC32[2];
            
    Long64_t nbytes0 = 0;
    // Start of Ttree loop
    for (Long64_t j=0; j<nentries0;j++) {
      
        nbytes0 += tree->GetEntry(j);
        
        E_HPGe=0; E_pyramid=0;
        
        HPGe_trigger=0; Pyramid_trigger=0;
        
        for (Long64_t k=0; k<2; k++){ // initializing to 0 for each event
              Energy_MADC32[k]=0;
        } // end for to initialize variables
        
        // Fill multiplicity histograms
        
        if(MADC32_1_Multiplicity != 0){ MADC32_mul -> Fill(MADC32_1_Multiplicity);}
        
        // Scheme of detectors channels:
        // HPGe: MADC32[1]
        // Pyramid: MADC32[0]
        
        // Check which detectors trigger
        if(MADC32_1_Multiplicity != 0){for (Int_t k=0; k<MADC32_1_Multiplicity; k++){if (MADC32_1_Channels[k] == 0){Pyramid_trigger=1;}}}
        if(MADC32_1_Multiplicity != 0){for (Int_t k=0; k<MADC32_1_Multiplicity; k++){if (MADC32_1_Channels[k] == 1){HPGe_trigger=1;}}}

        ///////////////
        // DETECTORS //
        ///////////////
        
        if (Pyramid_trigger == 1){ // 
          for (Long64_t j3=0; j3<MADC32_1_Multiplicity; j3++) {
              if (MADC32_1_Channels[j3] == 0){
              E_pyramid = MADC32_1_Values[j3];
              Scint[MADC32_1_Channels[j3]] -> Fill(E_pyramid);
            }
          }
        } // end MADC32_1
        
        if (HPGe_trigger == 1){ // 
          for (Long64_t j4=0; j4<MADC32_1_Multiplicity; j4++) {
              if (MADC32_1_Channels[j4] == 1){
              E_HPGe = MADC32_1_Values[j4];
              Scint[MADC32_1_Channels[j4]] -> Fill(E_HPGe);
            }
          }
        } // end MADC32_1
        
        if(totalEvents%1000000 == 0)cout<<totalEvents/1e6<<" millions read"<<endl;
        totalEvents ++;
        
    } // End LOOP OVER TTREE EVENTS

    cout<<"HPGe and Pyramid histograms filled"<<endl;

////////////////////////////////////////////////////////////
//                 Opening file to write                  // 
//////////////////////////////////////////////////////////// 
    
    cout << " Opening output file " << endl;
    TFile *output_file = new TFile("Output_1.root", "recreate");
        
    TDirectory *multiplicity = output_file->mkdir("Multiplicity");
    
    TDirectory *No_calib = output_file->mkdir("No_calibrated");
    TDirectory *MADC32_dir = No_calib->mkdir("MADC32");
    
    
    cout << " Output file opened " << endl;
   
////////////////////////////////////////////////////////////
//                  Writing output file                   // 
////////////////////////////////////////////////////////////
   
   for (Long64_t k=0; k<2;k++) {
            MADC32_dir->cd();
            Scint[k]->Write();
    }
    
    // Multiplicity
    multiplicity -> cd();
    MADC32_mul -> Write();
    
   output_file -> Close();
   cout<<"All histograms saved in .root file"<<endl;
   
   cout << "Program finished at:  " << endl; 
   system("/bin/date"); 
   cout << endl;
   
   return 0;
}

void analyze(){
    main();
}
