#include <stdio.h>
#include <string.h>
#include <iostream>     // std::cout, std::end
#include <fstream>

#include "TFile.h"
#include "TTree.h"
#include "TH1D.h"

////////////////////////////
// Initializing variables //
////////////////////////////

using std::ifstream;

// To compile:
// g++ mvme_unpacker.cpp `root-config --cflags --libs` -o mvme_unpacker.out
// To execute:
// ./mvme_unpacker.out inputfile.mvlclst outputfile.root

// If you want to customize this program with a different number of modules you should add new variables, definitions, initialization
// and add your modules with same order they appear in MVMe software in readFrame function. 
// If you want to add different modules than Caen785, MDPP-16 or Caen1190A you have to create new functions to read those modules.

using namespace std;

//////////////////////////////////////////////////
// All variables initialization, event and tree //
//////////////////////////////////////////////////
// Checking
ULong_t bytes_counter = 0;
ULong_t totalEvents = 0;
Int_t wrong_module_numbers = 0;
Int_t continue_frame_counter = 0;
Int_t broken_event_count = 0;
Int_t EOF_MARKER = 0;
Int_t DATAFRAME_marker = 0;

// MADC32 1
Int_t   MADC32_1_Multiplicity;
Int_t   MADC32_1_Channel[32*3];
ULong_t MADC32_1_Value[32*3];
ULong_t MADC32_1_Extended_timestamp;
ULong_t MADC32_1_Event_timestamp;

TH1F *MADC32_1_histo[32];
char MADC32_1_name[64], MADC32_1_histo_name[64];

//Histogram LOOP
void histoLOOP(){
  for (Int_t i=0; i<32; i++) {
    sprintf(MADC32_1_name,"MADC32_1_%i",i);
    sprintf(MADC32_1_histo_name,"MADC32_1_%i ; Channel ; Counts",i);
    MADC32_1_histo[i] = new TH1F(MADC32_1_name, MADC32_1_histo_name, 2*4096, 0, 2*4096);
  }
}

// Very large buffer as f->read needs a defined variable to storage what reads. Only used in jumpFrame and jumpModule.
char discard[4 * 65536];

// TFile
TFile *treeFile;

// TTree
TTree *EventTree; 

// initializes all data before an event  
void initEvent() 
{
  // Module 1                                   
  MADC32_1_Multiplicity = 0;                  
  MADC32_1_Extended_timestamp = 0;       
  MADC32_1_Event_timestamp = 0;          
  
  for (int i=0; i<128; i++) {
    if (i<32*3){
      // Module 1                         
      MADC32_1_Channel[i] = 0;           
      MADC32_1_Value[i] = 0;             
    }
  }
}

// Generate TTree structure
void createTree() 
{
  // MADC32_1
  EventTree->Branch("MADC32_1_Multiplicity",                 &MADC32_1_Multiplicity,                "MADC32_1_Multiplicity/I");
  EventTree->Branch("MADC32_1_Channels",                      MADC32_1_Channel,                     "MADC32_1_Channels[MADC32_1_Multiplicity]/I");
  EventTree->Branch("MADC32_1_Values",                        MADC32_1_Value,                       "MADC32_1_Values[MADC32_1_Multiplicity]/L");
  EventTree->Branch("MADC32_1_Extended_timestamps",          &MADC32_1_Extended_timestamp,          "MADC32_1_Extended_timestamps/L");
  EventTree->Branch("MADC32_1_Event_timestamps",             &MADC32_1_Event_timestamp,             "MADC32_1_Event_timestamps/L");
}

///////////////////////
// Functions to jump //
///////////////////////
// There are some types of frames or some modules that we want to jump over

void jumpFrame(ifstream *f, unsigned char frame_header[4]) 
{
//     cout  << to_string(frame_header[0]) << " " << to_string(frame_header[1]) << " " << to_string(frame_header[2]) << " " << to_string(frame_header[3]) << endl;
    
  int words_length = (frame_header[0] + (frame_header[1] << 8) ) & 0b0001111111111111;
//   cout << "Words to jump " << words_length << endl;
  
  if (words_length != 0) {
    f->read(discard, words_length * 4);
  }
  bytes_counter += (words_length * 4);
}

void jumpModule(ifstream *f)
{
  unsigned char block_read_header[4];
  f->read((char*) block_read_header, 4);
    
  unsigned short words_length = ( block_read_header[0] +  
                                (block_read_header[1] << 8) ) & 0b0001111111111111;
    
  if (words_length != 0) {
    f->read(discard, words_length * 4);
  }
}   

//////////////////////////////////
// Functions to read MADC-32 //
//////////////////////////////////
// Reading each 32-bit data words, if the number of channels that have a count 
// is odd,  mvme adds a 00 00 00 00 word before End of Event.

void readMADC32_data(ifstream *f, int module)
{
  unsigned char madc32_data[4];
  f->read((char*) madc32_data, 4);
    
  unsigned char madc32_data_check = (madc32_data[2] + (madc32_data[3] << 8) & 0b1111111111100000) >> 5;
    
  if (madc32_data_check == 0b00000100000) { 
    if (((madc32_data[0] + (madc32_data[1] << 8) + (madc32_data[2] << 16) + (madc32_data[3] << 24)) != 0)) {
            
      unsigned int channel = madc32_data[2] & 0b00011111;
      unsigned long value = ( madc32_data[0] + (madc32_data[1] << 8) ) & 0b0001111111111111;
            
      if ((module == 1) && (channel >= 0) && (channel <= 31)){
        MADC32_1_Channel[MADC32_1_Multiplicity] = channel;
        MADC32_1_Value[MADC32_1_Multiplicity] = value;
        MADC32_1_Multiplicity++;
        MADC32_1_histo[channel] -> Fill(value);
        if (MADC32_1_Multiplicity > 32*3-1) { MADC32_1_Multiplicity = 32*3-1; }
      }
    } // != 0
  }
  
  if(madc32_data_check == 0b00000100100){
    if(module == 1){
      MADC32_1_Extended_timestamp = (madc32_data[0] + (madc32_data[1] << 8) ) & 0b1111111111111111;
    }
  }
  
  if (((madc32_data[3] >> 6) & 0b11) == 3){ // Reading timestamp in the end of event
    if(module == 1){
      MADC32_1_Event_timestamp = (madc32_data[0] + (madc32_data[1] << 8) + (madc32_data[2] << 16) + ((madc32_data[3] & 0b00111111) << 24));
      }
    }
}

void readMADC32(ifstream *f, int module) 
{
  unsigned char block_read_header[4];
  f->read((char*) block_read_header, 4); // should be Type = 0xf5
  unsigned short module_event_length = (block_read_header[0] + (block_read_header[1] << 8) ) & 0b0001111111111111;
    
  if (module_event_length > 1) {
    unsigned char module_header[4];
    f->read((char*) module_header, 4);
    for (int i=0; i < module_event_length-1; i++) {
      readMADC32_data(f, module);
    }
  } else if (module_event_length == 1) {
        unsigned char dontCare[4];
        f->read((char*) dontCare, 4);
        broken_event_count++;
    }
}

//////////////////////////////////////
// Functions to read each dataFrame //
//////////////////////////////////////
// reads a frame, that usually contains an event  

void readFrame(ifstream *f) 
{
    unsigned char header[4];
    f->read((char*) header, 4);
    unsigned char frameType = header[3];
    bytes_counter += 4;
    DATAFRAME_marker = 0;
//     cout << "HEADER: "<< std::hex << (int) header[0] << " " << (int) header[1] << " " << (int) header[2] << " " << (int) header[3] << " " << endl;
    
    
    if (frameType == 0xFA) { 
//         cout << " Frame Type is: " << std::hex << (int) frameType << endl;
        // Those frames don't have relevant info, but EOF_MARKER, so we jump it always.
        
        unsigned int frame_subtype;
        frame_subtype = (header[1] + (header[2] << 8)) >> 5 & 0b1111111;
//         cout << " Frame sub-Type is: " << std::hex << (int) frame_subtype << endl;
        
        if (frame_subtype == 0x77) { // EOF_MARKER subtype
            cout <<  "EOF_MARKER " << header << endl;
            EOF_MARKER = 1;
        } 
        else {
            jumpFrame(f, header);
        }
    }
    
    else if (frameType == 0xF3) {
        DATAFRAME_marker = 1;
        // This frame marks the start of an event, so keep reading. Always followed by 0xF5 frame. 
        // Sometimes if data contained in this frame are too large, 0xF3 frames split in some more, so 
        // we have to check the Continue bit. If Continue bit is set, the following frames will be 
        // StackContinuation Frames (0xF9). The last frame in the series will have Continue bit cleared.
      
//       if (totalEvents > 1485980) {
//             cout << "Event " << totalEvents << " Bytes counted " << bytes_counter << endl;
//             cout << "HEADER: "<< std::hex << (int) header[0] << " " << (int) header[1] << " " << (int) header[2] << " " << (int) header[3] << " " << endl;
//         }
      
        initEvent();
        readMADC32(f,1);
//         jumpModule(f);
      
        
        if (totalEvents % 1000000 == 0) {
            cout << " Events counted: " << totalEvents/1e6 << " millions" << endl;
        }
        
        if (header[2] >> 7 == 0b1) 
        {
            //cout << "continue frame" << endl;
            continue_frame_counter += 1;
        }
        
        totalEvents++;
        
        int frame_F3_length = (header[0] + (header[1] << 8) ) & 0b0001111111111111;
        bytes_counter += frame_F3_length * 4;
        
    }
    
    else if (!frameType){
      cout << "-- No EOF found so stop reading the file--" << endl;
      EOF_MARKER = 1;
    }
    
    else {
        // cout << " ERROR: unknown frame type: '" << (int) frameType << "'" << endl;
        // cout << "jumping frame" << endl;
        jumpFrame(f, header);
    }
}
 
///////////////////////////////////////////////
// Functions to read files and generate tree //
///////////////////////////////////////////////
// Main function. Opens the file and proceeds according 
// with the command-line parameters
// filename[1] must be input file name and filename[2] output file name

int main (int len, char* filename[]) 
{      
    ifstream fp;
    fp.open(filename[1], ios::in | ios::binary);
    printf("%s\n", filename[1]);
    
    char dontcare[8];
    fp.read(dontcare , 8);    // Jump over MVLC_USB ascii and littleE marker
    bytes_counter += 8;
    
    EventTree = new TTree("EventTree", "EventTree");
    treeFile = new TFile(filename[2], "recreate");    
    createTree();
    histoLOOP();

    while (EOF_MARKER == 0) {
//     while(!fp.eof()){
//     for (int w=0; w < 8; w++) {
        readFrame(&fp);
        if(DATAFRAME_marker == 1){EventTree->Fill();}
        if(fp.eof()){EOF_MARKER=1;}
//         cout << "Bytes counted: " << bytes_counter << endl;
    }
        
    cout << "File read, EOF_MARKER checked, total events: " << totalEvents << endl;
    cout << "Events with unexpected module numbers: " << wrong_module_numbers << endl;    
    cout << "Events with continue frames: " << continue_frame_counter << endl;
    cout << "Broken events at caen (sum of all): " <<  broken_event_count << endl;

    //save the Tree header. The file will be automatically closed
    //when going out of the function scope
    EventTree->Write();
    TDirectory *MADC32_1_dir = treeFile->mkdir("MADC32_1");
    
    for (int i=0; i<32; i++){
      MADC32_1_dir->cd();
      MADC32_1_histo[i] -> Write();
    }
        
    cout << "done." << endl;
    treeFile->Close();
    fp.close();   
    
}
