///////////////////////////////////////////////////////////////////////////////
//           ANALYSIS PROGRAM FOR CMAM SETUP                                 // 
//                                            Vicente García Távora Sept 2022//
///////////////////////////////////////////////////////////////////////////////
// 4*32 DSSD + 14*4 SiBall + 4* PADs + 8*GAGGs + CEPA4

//C,C++ Libraries
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <cstdlib>
#include <fstream>
#include <algorithm> // for std::find
#include <iterator> // for std::begin, std::end
// ROOT libraries
#include <TROOT.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TTree.h>
#include <TFile.h>
#include <TF1.h>
#include <TRandom3.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TColor.h>
#include <TStyle.h>
#include <TCutG.h>
using namespace std;

Int_t main(){

    cout << "Program started at:  " << endl; 
    system("/bin/date"); 
    cout << endl;
    
////////////////////////////////////////////////////////////
//               Empty histograms generator               // 
////////////////////////////////////////////////////////////

// Defining histogram name variable
    char MDPP32_1_name[40], MDPP32_1_name_histo[40], MDPP32_2_name[40], MDPP32_2_name_histo[40], MDPP32_3_name[40], MDPP32_3_name_histo[40], MDPP32_4_name[40], MDPP32_4_name_histo[40];
    
// Defining histogram arrays
    TH1F *MDPP32_1[32], *MDPP32_2[32], *MDPP32_3[32], *MDPP32_4[32]; // DSSDs
        
// Loop to create empty histograms
    cout << "Creating empty histograms" << endl;
    for (Int_t i=0; i<32;i++) {
        sprintf(MDPP32_1_name,"MDPP32_1_%i",i);
        sprintf(MDPP32_1_name_histo,"MDPP32_1_%i ; Channel ; Counts",i);
        MDPP32_1[i] = new TH1F(MDPP32_1_name, MDPP32_1_name_histo, 65536, 0, 65536);
        
        sprintf(MDPP32_2_name,"MDPP32_2_%i",i);
        sprintf(MDPP32_2_name_histo,"MDPP32_2_%i ; Channel ; Counts",i);
        MDPP32_2[i] = new TH1F(MDPP32_2_name, MDPP32_2_name_histo, 65536, 0, 65536);
        
        sprintf(MDPP32_3_name,"MDPP32_3_%i",i);
        sprintf(MDPP32_3_name_histo,"MDPP32_3_%i ; Channel ; Counts",i);
        MDPP32_3[i] = new TH1F(MDPP32_3_name, MDPP32_3_name_histo, 65536, 0, 65536);
        
        sprintf(MDPP32_4_name,"MDPP32_4_%i",i);
        sprintf(MDPP32_4_name_histo,"MDPP32_4_%i ; Channel ; Counts",i);
        MDPP32_4[i] = new TH1F(MDPP32_4_name, MDPP32_4_name_histo, 65536, 0, 65536);
    }
    cout << "OK Empty histograms created" << endl;

////////////////////////////////////////////////////////////
//             Declaring TTree and Variables              // 
//////////////////////////////////////////////////////////// 
    
// Connect tree file
    
    ifstream finp("inputlist.txt");
    string fileinput;
    TChain* tree = new TChain("EventTree");
   
    while(finp>>fileinput){
    tree -> Add(fileinput.c_str());
    cout<<"Input file: "<<fileinput<<endl;
    }
    
//Declaration of leaves types
   
    Int_t   MDPP32_1_Multiplicity;
    Int_t   MDPP32_1_Channels[32*3];
    Long64_t MDPP32_1_Values[32*3];
    Int_t   MDPP32_1_TDC_time_diff_mul;
    Int_t   MDPP32_1_TDC_time_diff_Channels[32*3];
    Long64_t MDPP32_1_TDC_time_diff_Values[32*3];
    Int_t   MDPP32_1_Trigger_time_diff_mul;
    Int_t   MDPP32_1_Trigger_time_diff_Channels[32];
    Long64_t MDPP32_1_Trigger_time_diff_Values[32];
    Long64_t MDPP32_1_Extended_timestamps;
    Long64_t MDPP32_1_Event_timestamps;
   
    Int_t   MDPP32_2_Multiplicity;
    Int_t   MDPP32_2_Channels[32*3];
    Long64_t MDPP32_2_Values[32*3];
    Int_t   MDPP32_2_TDC_time_diff_mul;
    Int_t   MDPP32_2_TDC_time_diff_Channels[32*3];
    Long64_t MDPP32_2_TDC_time_diff_Values[32*3];
    Int_t   MDPP32_2_Trigger_time_diff_mul;
    Int_t   MDPP32_2_Trigger_time_diff_Channels[32];
    Long64_t MDPP32_2_Trigger_time_diff_Values[32];
    Long64_t MDPP32_2_Extended_timestamps;
    Long64_t MDPP32_2_Event_timestamps;
   
    Int_t   MDPP32_3_Multiplicity;
    Int_t   MDPP32_3_Channels[32*3];
    Long64_t MDPP32_3_Values[32*3];
    Int_t   MDPP32_3_TDC_time_diff_mul;
    Int_t   MDPP32_3_TDC_time_diff_Channels[32*3];
    Long64_t MDPP32_3_TDC_time_diff_Values[32*3];
    Int_t   MDPP32_3_Trigger_time_diff_mul;
    Int_t   MDPP32_3_Trigger_time_diff_Channels[32];
    Long64_t MDPP32_3_Trigger_time_diff_Values[32];
    Long64_t MDPP32_3_Extended_timestamps;
    Long64_t MDPP32_3_Event_timestamps;

    Int_t   MDPP32_4_Multiplicity;
    Int_t   MDPP32_4_Channels[32*3];
    Long64_t MDPP32_4_Values[32*3];
    Int_t   MDPP32_4_TDC_time_diff_mul;
    Int_t   MDPP32_4_TDC_time_diff_Channels[32*3];
    Long64_t MDPP32_4_TDC_time_diff_Values[32*3];
    Int_t   MDPP32_4_Trigger_time_diff_mul;
    Int_t   MDPP32_4_Trigger_time_diff_Channels[32];
    Long64_t MDPP32_4_Trigger_time_diff_Values[32];
    Long64_t MDPP32_4_Extended_timestamps;
    Long64_t MDPP32_4_Event_timestamps;

   // Set branch addresses.
   tree->SetBranchAddress("MDPP32_1_Multiplicity",&MDPP32_1_Multiplicity);
   tree->SetBranchAddress("MDPP32_1_Channels",MDPP32_1_Channels);
   tree->SetBranchAddress("MDPP32_1_Values",MDPP32_1_Values);
   tree->SetBranchAddress("MDPP32_1_TDC_time_diff_mul",&MDPP32_1_TDC_time_diff_mul);
   tree->SetBranchAddress("MDPP32_1_TDC_time_diff_Channels",MDPP32_1_TDC_time_diff_Channels);
   tree->SetBranchAddress("MDPP32_1_TDC_time_diff_Values",MDPP32_1_TDC_time_diff_Values);
   tree->SetBranchAddress("MDPP32_1_Trigger_time_diff_mul",&MDPP32_1_Trigger_time_diff_mul);
   tree->SetBranchAddress("MDPP32_1_Trigger_time_diff_Channels",MDPP32_1_Trigger_time_diff_Channels);
   tree->SetBranchAddress("MDPP32_1_Trigger_time_diff_Values",MDPP32_1_Trigger_time_diff_Values);
   tree->SetBranchAddress("MDPP32_1_Extended_timestamps",&MDPP32_1_Extended_timestamps);
   tree->SetBranchAddress("MDPP32_1_Event_timestamps",&MDPP32_1_Event_timestamps);
   
   tree->SetBranchAddress("MDPP32_2_Multiplicity",&MDPP32_2_Multiplicity);
   tree->SetBranchAddress("MDPP32_2_Channels",MDPP32_2_Channels);
   tree->SetBranchAddress("MDPP32_2_Values",MDPP32_2_Values);
   tree->SetBranchAddress("MDPP32_2_TDC_time_diff_mul",&MDPP32_2_TDC_time_diff_mul);
   tree->SetBranchAddress("MDPP32_2_TDC_time_diff_Channels",MDPP32_2_TDC_time_diff_Channels);
   tree->SetBranchAddress("MDPP32_2_TDC_time_diff_Values",MDPP32_2_TDC_time_diff_Values);
   tree->SetBranchAddress("MDPP32_2_Trigger_time_diff_mul",&MDPP32_2_Trigger_time_diff_mul);
   tree->SetBranchAddress("MDPP32_2_Trigger_time_diff_Channels",MDPP32_2_Trigger_time_diff_Channels);
   tree->SetBranchAddress("MDPP32_2_Trigger_time_diff_Values",MDPP32_2_Trigger_time_diff_Values);
   tree->SetBranchAddress("MDPP32_2_Extended_timestamps",&MDPP32_2_Extended_timestamps);
   tree->SetBranchAddress("MDPP32_2_Event_timestamps",&MDPP32_2_Event_timestamps);
   
   tree->SetBranchAddress("MDPP32_3_Multiplicity",&MDPP32_3_Multiplicity);
   tree->SetBranchAddress("MDPP32_3_Channels",MDPP32_3_Channels);
   tree->SetBranchAddress("MDPP32_3_Values",MDPP32_3_Values);
   tree->SetBranchAddress("MDPP32_3_TDC_time_diff_mul",&MDPP32_3_TDC_time_diff_mul);
   tree->SetBranchAddress("MDPP32_3_TDC_time_diff_Channels",MDPP32_3_TDC_time_diff_Channels);
   tree->SetBranchAddress("MDPP32_3_TDC_time_diff_Values",MDPP32_3_TDC_time_diff_Values);
   tree->SetBranchAddress("MDPP32_3_Trigger_time_diff_mul",&MDPP32_3_Trigger_time_diff_mul);
   tree->SetBranchAddress("MDPP32_3_Trigger_time_diff_Channels",MDPP32_3_Trigger_time_diff_Channels);
   tree->SetBranchAddress("MDPP32_3_Trigger_time_diff_Values",MDPP32_3_Trigger_time_diff_Values);
   tree->SetBranchAddress("MDPP32_3_Extended_timestamps",&MDPP32_3_Extended_timestamps);
   tree->SetBranchAddress("MDPP32_3_Event_timestamps",&MDPP32_3_Event_timestamps);
   
   tree->SetBranchAddress("MDPP32_4_Multiplicity",&MDPP32_4_Multiplicity);
   tree->SetBranchAddress("MDPP32_4_Channels",MDPP32_4_Channels);
   tree->SetBranchAddress("MDPP32_4_Values",MDPP32_4_Values);
   tree->SetBranchAddress("MDPP32_4_TDC_time_diff_mul",&MDPP32_4_TDC_time_diff_mul);
   tree->SetBranchAddress("MDPP32_4_TDC_time_diff_Channels",MDPP32_4_TDC_time_diff_Channels);
   tree->SetBranchAddress("MDPP32_4_TDC_time_diff_Values",MDPP32_4_TDC_time_diff_Values);
   tree->SetBranchAddress("MDPP32_4_Trigger_time_diff_mul",&MDPP32_4_Trigger_time_diff_mul);
   tree->SetBranchAddress("MDPP32_4_Trigger_time_diff_Channels",MDPP32_4_Trigger_time_diff_Channels);
   tree->SetBranchAddress("MDPP32_4_Trigger_time_diff_Values",MDPP32_4_Trigger_time_diff_Values);
   tree->SetBranchAddress("MDPP32_4_Extended_timestamps",&MDPP32_4_Extended_timestamps);
   tree->SetBranchAddress("MDPP32_4_Event_timestamps",&MDPP32_4_Event_timestamps);
                           
////////////////////////////////////////////////////////////
//           Conditions histograms & variables            // 
//////////////////////////////////////////////////////////// 

// Study the multiplicity of our events on our detectors and modules
// Define our multiplicity histograms for each MDPP32 module
    TH1F *MDPP32_1_mul = new TH1F("MDPP32_1_multiplicity", "MDPP32_1 multiplicity ; Event multiplicity ; Counts",32*3, 0, 32*3-1);
    TH1F *MDPP32_2_mul = new TH1F("MDPP32_2_multiplicity", "MDPP32_2 multiplicity ; Event multiplicity ; Counts",32*3, 0, 32*3-1);
    TH1F *MDPP32_3_mul = new TH1F("MDPP32_3_multiplicity", "MDPP32_3 multiplicity ; Event multiplicity ; Counts",32*3, 0, 32*3-1);
    TH1F *MDPP32_4_mul = new TH1F("MDPP32_4_multiplicity", "MDPP32_4 multiplicity ; Event multiplicity ; Counts",32*3, 0, 32*3-1);

////////////////////////////////////////////////////////////
//                  LOOP OVER OUR TTREE                   // 
////////////////////////////////////////////////////////////    

// We are going to apply some conditions to our data and fill our histograms
    Long64_t nentries0 = tree->GetEntries();
    cout<< "Filling DSSD, CEPA and 2D histograms" << endl;
    cout<< " Total events: "<< nentries0/1e6 << " millions"<<endl;
    Long64_t totalEvents=0;
    
    Double_t Emdpp32_1, Emdpp32_2, Emdpp32_3, Emdpp32_4;
    Int_t mdpp32_1_trigger, mdpp32_2_trigger, mdpp32_3_trigger, mdpp32_4_trigger;
    
    Long64_t nbytes0 = 0;
    // Start of Ttree loop
    for (Long64_t j=0; j<nentries0;j++) {
        nbytes0 += tree->GetEntry(j);
        
        Emdpp32_1=0;
        Emdpp32_2=0;
        Emdpp32_3=0;
        Emdpp32_4=0;
        
        mdpp32_1_trigger=0; mdpp32_1_trigger=0; mdpp32_1_trigger=0; mdpp32_1_trigger=0;
        
        // Fill multiplicity histograms
        if(MDPP32_1_Multiplicity != 0){MDPP32_1_mul -> Fill(MDPP32_1_Multiplicity);}
        if(MDPP32_2_Multiplicity != 0){MDPP32_2_mul -> Fill(MDPP32_2_Multiplicity);}
        if(MDPP32_3_Multiplicity != 0){MDPP32_3_mul -> Fill(MDPP32_3_Multiplicity);}
        if(MDPP32_4_Multiplicity != 0){MDPP32_4_mul -> Fill(MDPP32_4_Multiplicity);}
        
        // Check which detectors trigger
        if(MDPP32_1_Multiplicity != 0){mdpp32_1_trigger=1;}
        if(MDPP32_2_Multiplicity != 0){mdpp32_2_trigger=1;}
        if(MDPP32_3_Multiplicity != 0){mdpp32_3_trigger=1;}
        if(MDPP32_4_Multiplicity != 0){mdpp32_4_trigger=1;}

        ///////////////
        // DETECTORS //
        ///////////////
        
        if (mdpp32_1_trigger == 1){ // Check if detector triggers
          for (Long64_t j0=0; j0<MDPP32_1_Multiplicity; j0++) {
            Emdpp32_1 = MDPP32_1_Values[j0];
            MDPP32_1[MDPP32_1_Channels[j0]] -> Fill(Emdpp32_1);
          }
        }// end MDPP32_1
        
        if (mdpp32_2_trigger == 1){ // Check if detector triggers
          for (Long64_t j1=0; j1<MDPP32_2_Multiplicity; j1++) {
            Emdpp32_2 = MDPP32_2_Values[j1];
            MDPP32_2[MDPP32_2_Channels[j1]] -> Fill(Emdpp32_2);
          }
        }// end MDPP32_2
          
          if (mdpp32_3_trigger == 1){ // Check if detector triggers
          for (Long64_t j2=0; j2<MDPP32_3_Multiplicity; j2++) {
            Emdpp32_3 = MDPP32_3_Values[j2];
            MDPP32_3[MDPP32_3_Channels[j2]] -> Fill(Emdpp32_3);
          }
          }// end MDPP32_3
          
          if (mdpp32_4_trigger == 1){ // Check if detector triggers
          for (Long64_t j3=0; j3<MDPP32_4_Multiplicity; j3++) {
            Emdpp32_4 = MDPP32_4_Values[j3];
            MDPP32_4[MDPP32_4_Channels[j3]] -> Fill(Emdpp32_4);
          }
          }// end MDPP32_4
        
        if(totalEvents%1000000 == 0)cout<<totalEvents/1e6<<" millions read"<<endl;
        totalEvents ++;
        
    } // End LOOP OVER TTREE EVENTS

    cout<<"DSSD, SiBall and CEPA histograms filled"<<endl;

////////////////////////////////////////////////////////////
//                 Opening file to write                  // 
//////////////////////////////////////////////////////////// 
    
    cout << " Opening output file " << endl;
    TFile *output_file = new TFile("TEAFN_Monolythic.root", "recreate");
        
    TDirectory *multiplicity = output_file->mkdir("Multiplicity");
    
    TDirectory *No_calib = output_file->mkdir("No_calibrated");
    TDirectory *MDPP32_1_dir = No_calib->mkdir("MDPP32_1");
    TDirectory *MDPP32_2_dir = No_calib->mkdir("MDPP32_2");
    TDirectory *MDPP32_3_dir = No_calib->mkdir("MDPP32_3");
    TDirectory *MDPP32_4_dir = No_calib->mkdir("MDPP32_4");
    
    cout << " Output file opened " << endl;
   
////////////////////////////////////////////////////////////
//                  Writing output file                   // 
////////////////////////////////////////////////////////////
   
   for (Long64_t k=0; k<32;k++) {
     
     MDPP32_1_dir->cd();
     MDPP32_1[k]->Write();
     
     MDPP32_2_dir->cd();
     MDPP32_2[k]->Write();
     
     MDPP32_3_dir->cd();
     MDPP32_4[k]->Write();
     
     MDPP32_4_dir->cd();
     MDPP32_4[k]->Write();
  }
    
// Multiplicity folder
    multiplicity -> cd();
    MDPP32_1_mul -> Write();
    MDPP32_2_mul -> Write();
    MDPP32_3_mul -> Write();
    MDPP32_4_mul -> Write();
    
   output_file -> Close();
   cout<<"All histograms saved in .root file"<<endl;
   
   cout << "Program finished at:  " << endl; 
   system("/bin/date"); 
   cout << endl;
   
   return 0;
}

void analyze(){
    main();
}
